=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-04-20 20:58:51 +0000
+++ bzrlib/branch.py	2009-04-23 16:02:28 +0000
@@ -2434,11 +2434,13 @@
         super(BzrBranch8, self).__init__(*args, **kwargs)
         self._last_revision_info_cache = None
         self._partial_revision_history_cache = []
+        self._reference_info = None
 
     def _clear_cached_state(self):
         super(BzrBranch8, self)._clear_cached_state()
         self._last_revision_info_cache = None
         self._partial_revision_history_cache = []
+        self._reference_info = None
 
     def _last_revision_info(self):
         revision_string = self._transport.get_bytes('last-revision')
@@ -2554,6 +2556,7 @@
         """Set the parent branch"""
         return self._get_config_location('parent_location')
 
+    @needs_write_lock
     def _set_all_reference_info(self, info_dict):
         """Replace all reference info stored in a branch.
 
@@ -2566,12 +2569,16 @@
                                 branch_location=branch_location)
             writer.write_stanza(stanza)
         self._transport.put_bytes('references', s.getvalue())
+        self._reference_info = info_dict
 
+    @needs_read_lock
     def _get_all_reference_info(self):
         """Return all the reference info stored in a branch.
 
         :return: A dict of {file_id: (tree_path, branch_location)}
         """
+        if self._reference_info is not None:
+            return self._reference_info
         rio_file = self._transport.get('references')
         try:
             stanzas = rio.read_stanzas(rio_file)
@@ -2579,6 +2586,7 @@
                              s['branch_location'])) for s in stanzas)
         finally:
             rio_file.close()
+        self._reference_info = info_dict
         return info_dict
 
     def set_reference_info(self, file_id, tree_path, branch_location):

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2009-04-14 15:29:01 +0000
+++ bzrlib/tests/test_branch.py	2009-04-23 16:02:28 +0000
@@ -379,6 +379,67 @@
         self.assertTrue(branch.repository.has_revision(revid))
 
 
+class BzrBranch8(TestCaseWithTransport):
+
+    def make_branch(self, location, format=None):
+        if format is None:
+            format = bzrdir.format_registry.make_bzrdir('1.9')
+            format.set_branch_format(_mod_branch.BzrBranchFormat8())
+        return TestCaseWithTransport.make_branch(self, location, format=format)
+
+    def create_branch_with_reference(self):
+        branch = self.make_branch('branch')
+        branch._set_all_reference_info({'file-id': ('path', 'location')})
+        return branch
+
+    @staticmethod
+    def instrument_branch(branch, gets):
+        old_get = branch._transport.get
+        def get(*args, **kwargs):
+            gets.append((args, kwargs))
+            return old_get(*args, **kwargs)
+        branch._transport.get = get
+
+    def test_reference_info_caching_read_locked(self):
+        gets = []
+        branch = self.create_branch_with_reference()
+        branch.lock_read()
+        self.addCleanup(branch.unlock)
+        self.instrument_branch(branch, gets)
+        branch.get_reference_info('file-id')
+        branch.get_reference_info('file-id')
+        self.assertEqual(1, len(gets))
+
+    def test_reference_info_caching_read_unlocked(self):
+        gets = []
+        branch = self.create_branch_with_reference()
+        self.instrument_branch(branch, gets)
+        branch.get_reference_info('file-id')
+        branch.get_reference_info('file-id')
+        self.assertEqual(2, len(gets))
+
+    def test_reference_info_caching_write_locked(self):
+        gets = []
+        branch = self.make_branch('branch')
+        branch.lock_write()
+        self.instrument_branch(branch, gets)
+        self.addCleanup(branch.unlock)
+        branch._set_all_reference_info({'file-id': ('path2', 'location2')})
+        path, location = branch.get_reference_info('file-id')
+        self.assertEqual(0, len(gets))
+        self.assertEqual('path2', path)
+        self.assertEqual('location2', location)
+
+    def test_reference_info_caches_cleared(self):
+        branch = self.make_branch('branch')
+        branch.lock_write()
+        branch.set_reference_info('file-id', 'path2', 'location2')
+        branch.unlock()
+        doppelganger = Branch.open('branch')
+        doppelganger.set_reference_info('file-id', 'path3', 'location3')
+        self.assertEqual(('path3', 'location3'),
+                         branch.get_reference_info('file-id'))
+
 class TestBranchReference(TestCaseWithTransport):
     """Tests for the branch reference facility."""
 

